/*
 Copyright (C) 2025 Hugo Huang, Farama Foundation, and the respective contributors

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
*/

#ifndef __VIZ_DOOM_CLASSES_H__
#define __VIZ_DOOM_CLASSES_H__

#include <string>
#include <vector>
#include <unordered_map>

// Doom class information auto-generated from ZDoom wiki.
// Generated by scripts/fetch_doom_classes.py

// Listing default object categories in Doom
const std::vector<std::string> categories = {
    "Ammo",
    "Armor",
    "Artifact",
    "Breakable",
    "Bridge",
    "Decoration",
    "DynamicLight",
    "Explosive",
    "Gibs",
    "Gore",
    "Hazard",
    "Health",
    "InteractiveObject",
    "Internal",
    "Key",
    "LightSource",
    "MapSpot",
    "Monster",
    "Player",
    "Power",
    "Powerup",
    "PuzzleItem",
    "QuestItem",
    "Self",
    "SFX",
    "ScriptThing",
    "Token",
    "Vegetation",
    "Weapon",
};

// Mapping from class names to their category
const std::unordered_map<std::string, std::string> classToCategory = {
    {"ammosatchel", "Ammo"},
    {"backpack", "Ammo"},
    {"bagofholding", "Ammo"},
    {"blasterammo", "Ammo"},
    {"blasterhefty", "Ammo"},
    {"boxofbullets", "Ammo"},
    {"cell", "Ammo"},
    {"cellpack", "Ammo"},
    {"clip", "Ammo"},
    {"clipbox", "Ammo"},
    {"clipofbullets", "Ammo"},
    {"crateofmissiles", "Ammo"},
    {"crossbowammo", "Ammo"},
    {"crossbowhefty", "Ammo"},
    {"electricbolts", "Ammo"},
    {"energypack", "Ammo"},
    {"energypod", "Ammo"},
    {"goldwandammo", "Ammo"},
    {"goldwandhefty", "Ammo"},
    {"hegrenaderounds", "Ammo"},
    {"largezorchpack", "Ammo"},
    {"largezorchrecharge", "Ammo"},
    {"maceammo", "Ammo"},
    {"macehefty", "Ammo"},
    {"mana1", "Ammo"},
    {"mana2", "Ammo"},
    {"mana3", "Ammo"},
    {"minimissiles", "Ammo"},
    {"minizorchpack", "Ammo"},
    {"minizorchrecharge", "Ammo"},
    {"phasingzorch", "Ammo"},
    {"phasingzorchpack", "Ammo"},
    {"phoenixrodammo", "Ammo"},
    {"phoenixrodhefty", "Ammo"},
    {"phosphorusgrenaderounds", "Ammo"},
    {"poisonbolts", "Ammo"},
    {"propulsorzorch", "Ammo"},
    {"propulsorzorchpack", "Ammo"},
    {"rocketammo", "Ammo"},
    {"rocketbox", "Ammo"},
    {"shell", "Ammo"},
    {"shellbox", "Ammo"},
    {"skullrodammo", "Ammo"},
    {"skullrodhefty", "Ammo"},
    {"zorchpack", "Ammo"},
    {"amuletofwarding", "Armor"},
    {"armorbonus", "Armor"},
    {"bluearmor", "Armor"},
    {"bluearmorformegasphere", "Armor"},
    {"chexarmor", "Armor"},
    {"enchantedshield", "Armor"},
    {"falconshield", "Armor"},
    {"greenarmor", "Armor"},
    {"leatherarmor", "Armor"},
    {"mesharmor", "Armor"},
    {"metalarmor", "Armor"},
    {"platinumhelm", "Armor"},
    {"silvershield", "Armor"},
    {"slimerepellent", "Armor"},
    {"superchexarmor", "Armor"},
    {"artiblastradius", "Artifact"},
    {"artiboostarmor", "Artifact"},
    {"artiboostmana", "Artifact"},
    {"artidarkservant", "Artifact"},
    {"artiegg", "Artifact"},
    {"artifly", "Artifact"},
    {"artihealingradius", "Artifact"},
    {"artihealth", "Artifact"},
    {"artiinvisibility", "Artifact"},
    {"artiinvulnerability", "Artifact"},
    {"artiinvulnerability2", "Artifact"},
    {"artipoisonbag", "Artifact"},
    {"artipoisonbag1", "Artifact"},
    {"artipoisonbag2", "Artifact"},
    {"artipoisonbag3", "Artifact"},
    {"artipoisonbaggiver", "Artifact"},
    {"artipoisonbagshooter", "Artifact"},
    {"artipork", "Artifact"},
    {"artispeedboots", "Artifact"},
    {"artisuperhealth", "Artifact"},
    {"artiteleport", "Artifact"},
    {"artiteleportother", "Artifact"},
    {"artitimebomb", "Artifact"},
    {"artitomeofpower", "Artifact"},
    {"artitorch", "Artifact"},
    {"environmentalsuit", "Artifact"},
    {"medpatch", "Artifact"},
    {"medicalkit", "Artifact"},
    {"scanner", "Artifact"},
    {"shadowarmor", "Artifact"},
    {"surgerykit", "Artifact"},
    {"teleporterbeacon", "Artifact"},
    {"computer", "Breakable"},
    {"piston", "Breakable"},
    {"pottery1", "Breakable"},
    {"pottery2", "Breakable"},
    {"pottery3", "Breakable"},
    {"powercoupling", "Breakable"},
    {"powercrystal", "Breakable"},
    {"treedestructible", "Breakable"},
    {"woodenbarrel", "Breakable"},
    {"zshrub1", "Breakable"},
    {"zshrub2", "Breakable"},
    {"zsuitofarmor", "Breakable"},
    {"zxmastree", "Breakable"},
    {"bridge", "Bridge"},
    {"custombridge", "Bridge"},
    {"invisiblebridge", "Bridge"},
    {"invisiblebridge16", "Bridge"},
    {"invisiblebridge32", "Bridge"},
    {"invisiblebridge8", "Bridge"},
    {"zbridge", "Bridge"},
    {"alienaspclimber", "Decoration"},
    {"ammofiller", "Decoration"},
    {"anvil", "Decoration"},
    {"barrel", "Decoration"},
    {"barricadecolumn", "Decoration"},
    {"brownpillar", "Decoration"},
    {"cavepillarbottom", "Decoration"},
    {"cavepillartop", "Decoration"},
    {"ceilingchain", "Decoration"},
    {"chexcaverncolumn", "Decoration"},
    {"chexcavernstalagmite", "Decoration"},
    {"chexchemicalburner", "Decoration"},
    {"chexchemicalflask", "Decoration"},
    {"chexcivilian1", "Decoration"},
    {"chexcivilian2", "Decoration"},
    {"chexcivilian3", "Decoration"},
    {"chexflagonpole", "Decoration"},
    {"chexgastank", "Decoration"},
    {"chexminecart", "Decoration"},
    {"chexslimefountain", "Decoration"},
    {"chexspaceship", "Decoration"},
    {"chimneystack", "Decoration"},
    {"entitynest", "Decoration"},
    {"heartcolumn", "Decoration"},
    {"keygizmoblue", "Decoration"},
    {"keygizmofloatblue", "Decoration"},
    {"keygizmofloatgreen", "Decoration"},
    {"keygizmofloatyellow", "Decoration"},
    {"keygizmogreen", "Decoration"},
    {"keygizmoyellow", "Decoration"},
    {"metalpot", "Decoration"},
    {"moss1", "Decoration"},
    {"moss2", "Decoration"},
    {"mug", "Decoration"},
    {"pillaralienpower", "Decoration"},
    {"pillaraztec", "Decoration"},
    {"pillaraztecdamaged", "Decoration"},
    {"pillaraztecruined", "Decoration"},
    {"pitcher", "Decoration"},
    {"pot", "Decoration"},
    {"rebelboots", "Decoration"},
    {"rebelhelmet", "Decoration"},
    {"rebelshirt", "Decoration"},
    {"rubble1", "Decoration"},
    {"rubble2", "Decoration"},
    {"rubble3", "Decoration"},
    {"rubble4", "Decoration"},
    {"rubble5", "Decoration"},
    {"rubble6", "Decoration"},
    {"rubble7", "Decoration"},
    {"rubble8", "Decoration"},
    {"srock1", "Decoration"},
    {"srock2", "Decoration"},
    {"srock3", "Decoration"},
    {"srock4", "Decoration"},
    {"sstalactitebig", "Decoration"},
    {"sstalactitesmall", "Decoration"},
    {"sstalagmitebig", "Decoration"},
    {"sstalagmitesmall", "Decoration"},
    {"shortgreencolumn", "Decoration"},
    {"shortredcolumn", "Decoration"},
    {"sigilbanner", "Decoration"},
    {"skullcolumn", "Decoration"},
    {"smallpillar", "Decoration"},
    {"smalltorchunlit", "Decoration"},
    {"stalactitelarge", "Decoration"},
    {"stalactitesmall", "Decoration"},
    {"stalagmite", "Decoration"},
    {"stalagmitelarge", "Decoration"},
    {"stalagmitesmall", "Decoration"},
    {"stalagtite", "Decoration"},
    {"statue", "Decoration"},
    {"statueruined", "Decoration"},
    {"stickinwater", "Decoration"},
    {"stool", "Decoration"},
    {"surgerycrab", "Decoration"},
    {"tableshit1", "Decoration"},
    {"tableshit10", "Decoration"},
    {"tableshit2", "Decoration"},
    {"tableshit3", "Decoration"},
    {"tableshit4", "Decoration"},
    {"tableshit5", "Decoration"},
    {"tableshit6", "Decoration"},
    {"tableshit7", "Decoration"},
    {"tableshit8", "Decoration"},
    {"tableshit9", "Decoration"},
    {"tallgreencolumn", "Decoration"},
    {"tallredcolumn", "Decoration"},
    {"techpillar", "Decoration"},
    {"tray", "Decoration"},
    {"tub", "Decoration"},
    {"waterbottle", "Decoration"},
    {"waterfountain", "Decoration"},
    {"zbannertattered", "Decoration"},
    {"zbarrel", "Decoration"},
    {"zbell", "Decoration"},
    {"zbucket", "Decoration"},
    {"zchainbit32", "Decoration"},
    {"zchainbit64", "Decoration"},
    {"zchainendhook1", "Decoration"},
    {"zchainendhook2", "Decoration"},
    {"zchainendspike", "Decoration"},
    {"zchandelierunlit", "Decoration"},
    {"zcorpsehanging", "Decoration"},
    {"zcorpselynched", "Decoration"},
    {"zironmaiden", "Decoration"},
    {"zlog", "Decoration"},
    {"zrock1", "Decoration"},
    {"zrock2", "Decoration"},
    {"zrock3", "Decoration"},
    {"zrock4", "Decoration"},
    {"zrockblack", "Decoration"},
    {"zrockbrown1", "Decoration"},
    {"zrockbrown2", "Decoration"},
    {"zrubble1", "Decoration"},
    {"zrubble2", "Decoration"},
    {"zrubble3", "Decoration"},
    {"zstalactiteicelarge", "Decoration"},
    {"zstalactiteicemedium", "Decoration"},
    {"zstalactiteicesmall", "Decoration"},
    {"zstalactiteicetiny", "Decoration"},
    {"zstalactitelarge", "Decoration"},
    {"zstalactitemedium", "Decoration"},
    {"zstalactitesmall", "Decoration"},
    {"zstalagmiteicelarge", "Decoration"},
    {"zstalagmiteicemedium", "Decoration"},
    {"zstalagmiteicesmall", "Decoration"},
    {"zstalagmiteicetiny", "Decoration"},
    {"zstalagmitelarge", "Decoration"},
    {"zstalagmitemedium", "Decoration"},
    {"zstalagmitepillar", "Decoration"},
    {"zstalagmitesmall", "Decoration"},
    {"zstatuegargoyleblueshort", "Decoration"},
    {"zstatuegargoylebluetall", "Decoration"},
    {"zstatuegargoyledarkredshort", "Decoration"},
    {"zstatuegargoyledarkredtall", "Decoration"},
    {"zstatuegargoylegreenshort", "Decoration"},
    {"zstatuegargoylegreentall", "Decoration"},
    {"zstatuegargoyleredshort", "Decoration"},
    {"zstatuegargoyleredtall", "Decoration"},
    {"zstatuegargoylerustshort", "Decoration"},
    {"zstatuegargoylerusttall", "Decoration"},
    {"zstatuegargoylestripetall", "Decoration"},
    {"zstatuegargoyletanshort", "Decoration"},
    {"zstatuegargoyletantall", "Decoration"},
    {"ztombstonebigcross", "Decoration"},
    {"ztombstonebrianp", "Decoration"},
    {"ztombstonebrianr", "Decoration"},
    {"ztombstonecrosscircle", "Decoration"},
    {"ztombstonerip", "Decoration"},
    {"ztombstoneshane", "Decoration"},
    {"ztombstonesmallcross", "Decoration"},
    {"zvasepillar", "Decoration"},
    {"zwingedstatue", "Decoration"},
    {"pointlight", "DynamicLight"},
    {"pointlightadditive", "DynamicLight"},
    {"pointlightflicker", "DynamicLight"},
    {"pointlightflickeradditive", "DynamicLight"},
    {"pointlightflickerrandom", "DynamicLight"},
    {"pointlightflickerrandomadditive", "DynamicLight"},
    {"pointlightflickerrandomsubtractive", "DynamicLight"},
    {"pointlightflickersubtractive", "DynamicLight"},
    {"pointlightpulse", "DynamicLight"},
    {"pointlightpulseadditive", "DynamicLight"},
    {"pointlightpulsesubtractive", "DynamicLight"},
    {"pointlightsubtractive", "DynamicLight"},
    {"sectorpointlight", "DynamicLight"},
    {"sectorpointlightadditive", "DynamicLight"},
    {"sectorpointlightsubtractive", "DynamicLight"},
    {"vavoomlightcolor", "DynamicLight"},
    {"vavoomlightwhite", "DynamicLight"},
    {"abaddonball", "Explosive"},
    {"activatedtimebomb", "Explosive"},
    {"arachnotronplasma", "Explosive"},
    {"archvilefire", "Explosive"},
    {"arrow", "Explosive"},
    {"axepuff", "Explosive"},
    {"axepuffglow", "Explosive"},
    {"bfg10kshot", "Explosive"},
    {"bfgball", "Explosive"},
    {"bfgextra", "Explosive"},
    {"baronball", "Explosive"},
    {"beakpuff", "Explosive"},
    {"beastball", "Explosive"},
    {"belphegorball", "Explosive"},
    {"bishopfx", "Explosive"},
    {"bishopmissile", "Explosive"},
    {"blasterfx1", "Explosive"},
    {"blasterpuff", "Explosive"},
    {"bulletpuff", "Explosive"},
    {"cflamemissile", "Explosive"},
    {"cstaffmissile", "Explosive"},
    {"cstaffpuff", "Explosive"},
    {"cacodemonball", "Explosive"},
    {"cacolanternball", "Explosive"},
    {"centaurfx", "Explosive"},
    {"circleflame", "Explosive"},
    {"crossbowfx1", "Explosive"},
    {"crossbowfx2", "Explosive"},
    {"crossbowfx3", "Explosive"},
    {"crusadermissile", "Explosive"},
    {"darkimpball", "Explosive"},
    {"dart", "Explosive"},
    {"demon1fx1", "Explosive"},
    {"demon2fx1", "Explosive"},
    {"doomimpball", "Explosive"},
    {"dragonexplosion", "Explosive"},
    {"dragonfireball", "Explosive"},
    {"eggfx", "Explosive"},
    {"electricbolt", "Explosive"},
    {"fswordflame", "Explosive"},
    {"fswordmissile", "Explosive"},
    {"fastflamemissile", "Explosive"},
    {"fatshot", "Explosive"},
    {"fireball", "Explosive"},
    {"firebomb", "Explosive"},
    {"firedemonmissile", "Explosive"},
    {"flamemissile", "Explosive"},
    {"frostmissile", "Explosive"},
    {"gauntletpuff1", "Explosive"},
    {"gauntletpuff2", "Explosive"},
    {"goldwandfx1", "Explosive"},
    {"goldwandfx2", "Explosive"},
    {"goldwandpuff1", "Explosive"},
    {"goldwandpuff2", "Explosive"},
    {"grenade", "Explosive"},
    {"hegrenade", "Explosive"},
    {"hammermissile", "Explosive"},
    {"hammerpuff", "Explosive"},
    {"headfx1", "Explosive"},
    {"headfx2", "Explosive"},
    {"headfx3", "Explosive"},
    {"hectshot", "Explosive"},
    {"hereticimpball", "Explosive"},
    {"holymissile", "Explosive"},
    {"holymissilepuff", "Explosive"},
    {"holypuff", "Explosive"},
    {"holyspirit", "Explosive"},
    {"hornrodfx1", "Explosive"},
    {"hornrodfx2", "Explosive"},
    {"iceguyfx", "Explosive"},
    {"iceguyfx2", "Explosive"},
    {"iceshard", "Explosive"},
    {"inquisitorshot", "Explosive"},
    {"knightaxe", "Explosive"},
    {"koraxbolt", "Explosive"},
    {"lazball", "Explosive"},
    {"lightning", "Explosive"},
    {"lightningceiling", "Explosive"},
    {"lightningfloor", "Explosive"},
    {"lightningzap", "Explosive"},
    {"loreshot", "Explosive"},
    {"macefx1", "Explosive"},
    {"macefx2", "Explosive"},
    {"macefx3", "Explosive"},
    {"macefx4", "Explosive"},
    {"magestafffx2", "Explosive"},
    {"magewandmissile", "Explosive"},
    {"maulerpuff", "Explosive"},
    {"maulertorpedo", "Explosive"},
    {"maulertorpedowave", "Explosive"},
    {"minimissile", "Explosive"},
    {"minotaurfx1", "Explosive"},
    {"minotaurfx2", "Explosive"},
    {"minotaurfx3", "Explosive"},
    {"mummyfx1", "Explosive"},
    {"phasezorchmissile", "Explosive"},
    {"phoenixfx1", "Explosive"},
    {"phoenixfx2", "Explosive"},
    {"phosphorousfire", "Explosive"},
    {"phosphorousgrenade", "Explosive"},
    {"plasmaball", "Explosive"},
    {"plasmaball1", "Explosive"},
    {"plasmaball2", "Explosive"},
    {"poisonbag", "Explosive"},
    {"poisonbolt", "Explosive"},
    {"poisoncloud", "Explosive"},
    {"poisondart", "Explosive"},
    {"porkfx", "Explosive"},
    {"projectileblade", "Explosive"},
    {"propulsormissile", "Explosive"},
    {"punchpuff", "Explosive"},
    {"rainpillar", "Explosive"},
    {"redaxe", "Explosive"},
    {"revenanttracer", "Explosive"},
    {"revenanttracersmoke", "Explosive"},
    {"ripper", "Explosive"},
    {"ripperball", "Explosive"},
    {"rocket", "Explosive"},
    {"sentinelfx1", "Explosive"},
    {"sentinelfx2", "Explosive"},
    {"serpentfx", "Explosive"},
    {"snakeproja", "Explosive"},
    {"snakeprojb", "Explosive"},
    {"snoutpuff", "Explosive"},
    {"sorcfx1", "Explosive"},
    {"sorcfx3", "Explosive"},
    {"sorcfx4", "Explosive"},
    {"sorcerer2fx1", "Explosive"},
    {"sorcerer2fx2", "Explosive"},
    {"sorcererfx1", "Explosive"},
    {"spawnshot", "Explosive"},
    {"spectrallightningball1", "Explosive"},
    {"spectrallightningball2", "Explosive"},
    {"spectrallightningbase", "Explosive"},
    {"spectrallightningbigball1", "Explosive"},
    {"spectrallightningbigball2", "Explosive"},
    {"spectrallightningbigv1", "Explosive"},
    {"spectrallightningbigv2", "Explosive"},
    {"spectrallightningdeath1", "Explosive"},
    {"spectrallightningdeath2", "Explosive"},
    {"spectrallightningdeathshort", "Explosive"},
    {"spectrallightningh1", "Explosive"},
    {"spectrallightningh2", "Explosive"},
    {"spectrallightningh3", "Explosive"},
    {"spectrallightninghtail", "Explosive"},
    {"spectrallightningspot", "Explosive"},
    {"spectrallightningv1", "Explosive"},
    {"spectrallightningv2", "Explosive"},
    {"staffpuff", "Explosive"},
    {"staffpuff2", "Explosive"},
    {"summoningdoll", "Explosive"},
    {"telotherfx1", "Explosive"},
    {"telotherfx2", "Explosive"},
    {"telotherfx3", "Explosive"},
    {"telotherfx4", "Explosive"},
    {"telotherfx5", "Explosive"},
    {"throwingbomb", "Explosive"},
    {"volcanoblast", "Explosive"},
    {"volcanotblast", "Explosive"},
    {"whirlwind", "Explosive"},
    {"wizardfx1", "Explosive"},
    {"wraithfx1", "Explosive"},
    {"alienchunklarge", "Gibs"},
    {"alienchunksmall", "Gibs"},
    {"axeblood", "Gibs"},
    {"blood", "Gibs"},
    {"bloodsplatter", "Gibs"},
    {"bloodyfighterskull", "Gibs"},
    {"bloodyskull", "Gibs"},
    {"centaurshield", "Gibs"},
    {"centaursword", "Gibs"},
    {"demon1chunk1", "Gibs"},
    {"demon1chunk2", "Gibs"},
    {"demon1chunk3", "Gibs"},
    {"demon1chunk4", "Gibs"},
    {"demon1chunk5", "Gibs"},
    {"demon2chunk1", "Gibs"},
    {"demon2chunk2", "Gibs"},
    {"demon2chunk3", "Gibs"},
    {"demon2chunk4", "Gibs"},
    {"demon2chunk5", "Gibs"},
    {"ettinmace", "Gibs"},
    {"feather", "Gibs"},
    {"firedemonsplotch1", "Gibs"},
    {"firedemonsplotch2", "Gibs"},
    {"firedroplet", "Gibs"},
    {"hereticimpchunk1", "Gibs"},
    {"hereticimpchunk2", "Gibs"},
    {"icechunkhead", "Gibs"},
    {"inquisitorarm", "Gibs"},
    {"junk", "Gibs"},
    {"koraxspirit", "Gibs"},
    {"meat", "Gibs"},
    {"mummysoul", "Gibs"},
    {"podgoo", "Gibs"},
    {"programmerbase", "Gibs"},
    {"realgibs", "Gibs"},
    {"serpentgib1", "Gibs"},
    {"serpentgib2", "Gibs"},
    {"serpentgib3", "Gibs"},
    {"serpenthead", "Gibs"},
    {"bloodpool", "Gore"},
    {"bloodytwitch", "Gore"},
    {"brainstem", "Gore"},
    {"colongibs", "Gore"},
    {"deadacolyte", "Gore"},
    {"deadcacodemon", "Gore"},
    {"deadcrusader", "Gore"},
    {"deaddemon", "Gore"},
    {"deaddoomimp", "Gore"},
    {"deadlostsoul", "Gore"},
    {"deadmarine", "Gore"},
    {"deadpeasant", "Gore"},
    {"deadreaver", "Gore"},
    {"deadrebel", "Gore"},
    {"deadshotgunguy", "Gore"},
    {"deadstick", "Gore"},
    {"deadstrifeplayer", "Gore"},
    {"deadzombieman", "Gore"},
    {"floatingskull", "Gore"},
    {"gibbedmarine", "Gore"},
    {"gibbedmarineextra", "Gore"},
    {"gibs", "Gore"},
    {"hangbnobrain", "Gore"},
    {"hangnoguts", "Gore"},
    {"hangtlookingdown", "Gore"},
    {"hangtlookingup", "Gore"},
    {"hangtnobrain", "Gore"},
    {"hangtskull", "Gore"},
    {"hangingcorpse", "Gore"},
    {"headonastick", "Gore"},
    {"headsonastick", "Gore"},
    {"heartsintank", "Gore"},
    {"livestick", "Gore"},
    {"meat2", "Gore"},
    {"meat3", "Gore"},
    {"meat4", "Gore"},
    {"meat5", "Gore"},
    {"nonsolidmeat2", "Gore"},
    {"nonsolidmeat3", "Gore"},
    {"nonsolidmeat4", "Gore"},
    {"nonsolidmeat5", "Gore"},
    {"nonsolidtwitch", "Gore"},
    {"pileofguts", "Gore"},
    {"sacrificedguy", "Gore"},
    {"skullhang35", "Gore"},
    {"skullhang45", "Gore"},
    {"skullhang60", "Gore"},
    {"skullhang70", "Gore"},
    {"smallbloodpool", "Gore"},
    {"tank1", "Gore"},
    {"tank2", "Gore"},
    {"tank3", "Gore"},
    {"tank4", "Gore"},
    {"tank5", "Gore"},
    {"tank6", "Gore"},
    {"zchainendheart", "Gore"},
    {"zchainendskull", "Gore"},
    {"zcorpsekabob", "Gore"},
    {"zcorpselynchednoheart", "Gore"},
    {"zcorpsesitting", "Gore"},
    {"zcorpsesleeping", "Gore"},
    {"degninore", "Hazard"},
    {"explosivebarrel", "Hazard"},
    {"explosivebarrel2", "Hazard"},
    {"pod", "Hazard"},
    {"thrustfloor", "Hazard"},
    {"thrustfloordown", "Hazard"},
    {"thrustfloorup", "Hazard"},
    {"volcano", "Hazard"},
    {"zpoisonshroom", "Hazard"},
    {"bowloffruit", "Health"},
    {"bowlofvegetables", "Health"},
    {"crystalvial", "Health"},
    {"glassofwater", "Health"},
    {"healthbonus", "Health"},
    {"medikit", "Health"},
    {"megaspherehealth", "Health"},
    {"soulsphere", "Health"},
    {"stimpack", "Health"},
    {"superchargebreakfast", "Health"},
    {"blueflag", "InteractiveObject"},
    {"blueskullst", "InteractiveObject"},
    {"goldflag", "InteractiveObject"},
    {"goldskullst", "InteractiveObject"},
    {"greenflag", "InteractiveObject"},
    {"greenskullst", "InteractiveObject"},
    {"redflag", "InteractiveObject"},
    {"redskullst", "InteractiveObject"},
    {"targetpractice", "InteractiveObject"},
    {"whiteflag", "InteractiveObject"},
    {"zcauldron", "InteractiveObject"},
    {"zcauldronunlit", "InteractiveObject"},
    {"zfirebull", "InteractiveObject"},
    {"zfirebullunlit", "InteractiveObject"},
    {"zgempedestal", "InteractiveObject"},
    {"ztwinedtorch", "InteractiveObject"},
    {"ztwinedtorchunlit", "InteractiveObject"},
    {"zwalltorch", "InteractiveObject"},
    {"zwalltorchunlit", "InteractiveObject"},
    {"zwingedstatuenoskull", "InteractiveObject"},
    {"actor", "Internal"},
    {"ammo", "Internal"},
    {"armor", "Internal"},
    {"backpackitem", "Internal"},
    {"basicarmor", "Internal"},
    {"basicarmorbonus", "Internal"},
    {"basicarmorpickup", "Internal"},
    {"clericweapon", "Internal"},
    {"clericweaponpiece", "Internal"},
    {"custominventory", "Internal"},
    {"custommonsterinvasionspot", "Internal"},
    {"custompickupinvasionspot", "Internal"},
    {"customsprite", "Internal"},
    {"dehackedpickup", "Internal"},
    {"demonchunk", "Internal"},
    {"doombuildercamera", "Internal"},
    {"doomkey", "Internal"},
    {"doomunusedstates", "Internal"},
    {"doomweapon", "Internal"},
    {"dummystrifeitem", "Internal"},
    {"dynamiclight", "Internal"},
    {"fakeinventory", "Internal"},
    {"fastprojectile", "Internal"},
    {"fighterweapon", "Internal"},
    {"fighterweaponpiece", "Internal"},
    {"flag", "Internal"},
    {"glassshard", "Internal"},
    {"health", "Internal"},
    {"healthpickup", "Internal"},
    {"heretickey", "Internal"},
    {"hereticweapon", "Internal"},
    {"hexenarmor", "Internal"},
    {"hexenkey", "Internal"},
    {"inventory", "Internal"},
    {"key", "Internal"},
    {"mageweapon", "Internal"},
    {"mageweaponpiece", "Internal"},
    {"maprevealer", "Internal"},
    {"maxhealth", "Internal"},
    {"morphprojectile", "Internal"},
    {"morphedmonster", "Internal"},
    {"particlefountain", "Internal"},
    {"playerchunk", "Internal"},
    {"playerpawn", "Internal"},
    {"powerup", "Internal"},
    {"powerupgiver", "Internal"},
    {"puzzleitem", "Internal"},
    {"questitem", "Internal"},
    {"raintracker", "Internal"},
    {"randomspawner", "Internal"},
    {"scoreitem", "Internal"},
    {"sectoraction", "Internal"},
    {"sectorflagsetter", "Internal"},
    {"skull", "Internal"},
    {"sorcball", "Internal"},
    {"speakericon", "Internal"},
    {"specialspot", "Internal"},
    {"spectatorcamera", "Internal"},
    {"spectralmonster", "Internal"},
    {"stackpoint", "Internal"},
    {"strifehumanoid", "Internal"},
    {"strifekey", "Internal"},
    {"strifeweapon", "Internal"},
    {"strifezap1", "Internal"},
    {"switchabledecoration", "Internal"},
    {"switchingdecoration", "Internal"},
    {"teamitem", "Internal"},
    {"unknown", "Internal"},
    {"vavoomlight", "Internal"},
    {"weapon", "Internal"},
    {"weapongiver", "Internal"},
    {"weaponholder", "Internal"},
    {"weaponpiece", "Internal"},
    {"basekey", "Key"},
    {"bluecard", "Key"},
    {"bluecrystalkey", "Key"},
    {"blueskull", "Key"},
    {"brasskey", "Key"},
    {"catacombkey", "Key"},
    {"chapelkey", "Key"},
    {"chexbluecard", "Key"},
    {"chexredcard", "Key"},
    {"chexyellowcard", "Key"},
    {"corekey", "Key"},
    {"factorykey", "Key"},
    {"goldkey", "Key"},
    {"govskey", "Key"},
    {"idbadge", "Key"},
    {"idcard", "Key"},
    {"keyaxe", "Key"},
    {"keyblue", "Key"},
    {"keycastle", "Key"},
    {"keycave", "Key"},
    {"keydungeon", "Key"},
    {"keyemerald", "Key"},
    {"keyfire", "Key"},
    {"keygreen", "Key"},
    {"keyhorn", "Key"},
    {"keyrusted", "Key"},
    {"keysilver", "Key"},
    {"keysteel", "Key"},
    {"keyswamp", "Key"},
    {"keyyellow", "Key"},
    {"maulerkey", "Key"},
    {"militaryid", "Key"},
    {"minekey", "Key"},
    {"newkey5", "Key"},
    {"oraclekey", "Key"},
    {"oraclepass", "Key"},
    {"orderkey", "Key"},
    {"passcard", "Key"},
    {"power1key", "Key"},
    {"power2key", "Key"},
    {"power3key", "Key"},
    {"prisonkey", "Key"},
    {"prisonpass", "Key"},
    {"redcard", "Key"},
    {"redcrystalkey", "Key"},
    {"redskull", "Key"},
    {"securitykey", "Key"},
    {"severedhand", "Key"},
    {"silverkey", "Key"},
    {"warehousekey", "Key"},
    {"yellowcard", "Key"},
    {"yellowskull", "Key"},
    {"alienbubblecolumn", "LightSource"},
    {"alienceilingbubble", "LightSource"},
    {"alienfloorbubble", "LightSource"},
    {"alienspiderlight", "LightSource"},
    {"bluetorch", "LightSource"},
    {"brasstorch", "LightSource"},
    {"burningbarrel", "LightSource"},
    {"burningbowl", "LightSource"},
    {"burningbrazier", "LightSource"},
    {"cagelight", "LightSource"},
    {"candelabra", "LightSource"},
    {"candle", "LightSource"},
    {"candlestick", "LightSource"},
    {"chandelier", "LightSource"},
    {"chexlandinglight", "LightSource"},
    {"chexlightcolumn", "LightSource"},
    {"column", "LightSource"},
    {"evileye", "LightSource"},
    {"firebrazier", "LightSource"},
    {"firething", "LightSource"},
    {"flamelarge", "LightSource"},
    {"flamelarge2", "LightSource"},
    {"flamelargetemp", "LightSource"},
    {"flamesmall", "LightSource"},
    {"flamesmall2", "LightSource"},
    {"flamesmalltemp", "LightSource"},
    {"greentorch", "LightSource"},
    {"headcandles", "LightSource"},
    {"hugetorch", "LightSource"},
    {"klaxonwarninglight", "LightSource"},
    {"largetorch", "LightSource"},
    {"lightbrownfluorescent", "LightSource"},
    {"lightglobe", "LightSource"},
    {"lightgoldfluorescent", "LightSource"},
    {"lightsilverfluorescent", "LightSource"},
    {"mediumtorch", "LightSource"},
    {"outsidelamp", "LightSource"},
    {"pillarhugetech", "LightSource"},
    {"pillartechno", "LightSource"},
    {"polelantern", "LightSource"},
    {"redtorch", "LightSource"},
    {"serpenttorch", "LightSource"},
    {"shortbluetorch", "LightSource"},
    {"shortgreentorch", "LightSource"},
    {"shortredtorch", "LightSource"},
    {"smalltorchlit", "LightSource"},
    {"strifeburningbarrel", "LightSource"},
    {"strifecandelabra", "LightSource"},
    {"techlamp", "LightSource"},
    {"techlamp2", "LightSource"},
    {"techlampbrass", "LightSource"},
    {"techlampsilver", "LightSource"},
    {"walltorch", "LightSource"},
    {"zbluecandle", "LightSource"},
    {"zcandle", "LightSource"},
    {"zchandelier", "LightSource"},
    {"ambientsound", "MapSpot"},
    {"ambientsoundnogravity", "MapSpot"},
    {"batspawner", "MapSpot"},
    {"bossspot", "MapSpot"},
    {"bosstarget", "MapSpot"},
    {"cajunbodynode", "MapSpot"},
    {"cajuntrace", "MapSpot"},
    {"colorsetter", "MapSpot"},
    {"entitypod", "MapSpot"},
    {"fs_mapspot", "MapSpot"},
    {"fadesetter", "MapSpot"},
    {"fogspawner", "MapSpot"},
    {"forcefieldguard", "MapSpot"},
    {"hereticsoundsequence1", "MapSpot"},
    {"hereticsoundsequence10", "MapSpot"},
    {"hereticsoundsequence2", "MapSpot"},
    {"hereticsoundsequence3", "MapSpot"},
    {"hereticsoundsequence4", "MapSpot"},
    {"hereticsoundsequence5", "MapSpot"},
    {"hereticsoundsequence6", "MapSpot"},
    {"hereticsoundsequence7", "MapSpot"},
    {"hereticsoundsequence8", "MapSpot"},
    {"hereticsoundsequence9", "MapSpot"},
    {"leafspawner", "MapSpot"},
    {"lowerstacklookonly", "MapSpot"},
    {"macespawner", "MapSpot"},
    {"mapmarker", "MapSpot"},
    {"mapspot", "MapSpot"},
    {"mapspotgravity", "MapSpot"},
    {"musicchanger", "MapSpot"},
    {"podgenerator", "MapSpot"},
    {"pointpuller", "MapSpot"},
    {"pointpusher", "MapSpot"},
    {"secactenter", "MapSpot"},
    {"secactexit", "MapSpot"},
    {"secacteyesabovec", "MapSpot"},
    {"secacteyesbelowc", "MapSpot"},
    {"secacteyesdive", "MapSpot"},
    {"secacteyessurface", "MapSpot"},
    {"secacthitceil", "MapSpot"},
    {"secacthitfakefloor", "MapSpot"},
    {"secacthitfloor", "MapSpot"},
    {"secactuse", "MapSpot"},
    {"secactusewall", "MapSpot"},
    {"sectorsilencer", "MapSpot"},
    {"skycamcompat", "MapSpot"},
    {"skypicker", "MapSpot"},
    {"skyviewpoint", "MapSpot"},
    {"soundenvironment", "MapSpot"},
    {"soundsequence", "MapSpot"},
    {"soundwaterfall", "MapSpot"},
    {"soundwind", "MapSpot"},
    {"soundwindhexen", "MapSpot"},
    {"teleglittergenerator1", "MapSpot"},
    {"teleglittergenerator2", "MapSpot"},
    {"teleportdest", "MapSpot"},
    {"teleportdest2", "MapSpot"},
    {"teleportdest3", "MapSpot"},
    {"upperstacklookonly", "MapSpot"},
    {"waterzone", "MapSpot"},
    {"zombiespawner", "MapSpot"},
    {"abaddon", "Monster"},
    {"acolyte", "Monster"},
    {"acolyteblue", "Monster"},
    {"acolytedgreen", "Monster"},
    {"acolytegold", "Monster"},
    {"acolytegray", "Monster"},
    {"acolytelgreen", "Monster"},
    {"acolytered", "Monster"},
    {"acolyterust", "Monster"},
    {"acolyteshadow", "Monster"},
    {"acolytetan", "Monster"},
    {"acolytetobe", "Monster"},
    {"alienspectre1", "Monster"},
    {"alienspectre2", "Monster"},
    {"alienspectre3", "Monster"},
    {"alienspectre4", "Monster"},
    {"alienspectre5", "Monster"},
    {"arachnotron", "Monster"},
    {"archvile", "Monster"},
    {"armoredflemoidusbipedicus", "Monster"},
    {"armorer", "Monster"},
    {"barkeep", "Monster"},
    {"baronofhell", "Monster"},
    {"beast", "Monster"},
    {"beggar", "Monster"},
    {"beggar1", "Monster"},
    {"beggar2", "Monster"},
    {"beggar3", "Monster"},
    {"beggar4", "Monster"},
    {"beggar5", "Monster"},
    {"belphegor", "Monster"},
    {"betaskull", "Monster"},
    {"bishop", "Monster"},
    {"blooddemon", "Monster"},
    {"bossbrain", "Monster"},
    {"bosseye", "Monster"},
    {"cacodemon", "Monster"},
    {"cacolantern", "Monster"},
    {"ceilingturret", "Monster"},
    {"centaur", "Monster"},
    {"centaurleader", "Monster"},
    {"centaurmash", "Monster"},
    {"chaingunguy", "Monster"},
    {"chexsoul", "Monster"},
    {"chicken", "Monster"},
    {"clericboss", "Monster"},
    {"clink", "Monster"},
    {"commanderkeen", "Monster"},
    {"crusader", "Monster"},
    {"cyberdemon", "Monster"},
    {"darkimp", "Monster"},
    {"demon", "Monster"},
    {"demon1", "Monster"},
    {"demon1mash", "Monster"},
    {"demon2", "Monster"},
    {"demon2mash", "Monster"},
    {"doomimp", "Monster"},
    {"dragon", "Monster"},
    {"entityboss", "Monster"},
    {"entitysecond", "Monster"},
    {"ettin", "Monster"},
    {"ettinmash", "Monster"},
    {"fatso", "Monster"},
    {"fighterboss", "Monster"},
    {"firedemon", "Monster"},
    {"flembrane", "Monster"},
    {"flemoidusbipedicus", "Monster"},
    {"flemoiduscommonus", "Monster"},
    {"flemoiduscycloptiscommonus", "Monster"},
    {"flemoiduscycloptiscommonusv3", "Monster"},
    {"flemoidusstridicus", "Monster"},
    {"hectebus", "Monster"},
    {"hellknight", "Monster"},
    {"heresiarch", "Monster"},
    {"hereticimp", "Monster"},
    {"hereticimpleader", "Monster"},
    {"iceguy", "Monster"},
    {"inquisitor", "Monster"},
    {"ironlich", "Monster"},
    {"kneelingguy", "Monster"},
    {"knight", "Monster"},
    {"knightghost", "Monster"},
    {"korax", "Monster"},
    {"loremaster", "Monster"},
    {"lostsoul", "Monster"},
    {"macil1", "Monster"},
    {"macil2", "Monster"},
    {"mageboss", "Monster"},
    {"marineberserk", "Monster"},
    {"marinebfg", "Monster"},
    {"marinechaingun", "Monster"},
    {"marinechainsaw", "Monster"},
    {"marinechainsawvzd", "Monster"},
    {"marinefist", "Monster"},
    {"marinepistol", "Monster"},
    {"marineplasma", "Monster"},
    {"marinerailgun", "Monster"},
    {"marinerocket", "Monster"},
    {"marineshotgun", "Monster"},
    {"marinessg", "Monster"},
    {"mbfhelperdog", "Monster"},
    {"medic", "Monster"},
    {"merchant", "Monster"},
    {"minotaur", "Monster"},
    {"minotaurfriend", "Monster"},
    {"mummy", "Monster"},
    {"mummyghost", "Monster"},
    {"mummyleader", "Monster"},
    {"mummyleaderghost", "Monster"},
    {"oracle", "Monster"},
    {"painelemental", "Monster"},
    {"peasant", "Monster"},
    {"peasant1", "Monster"},
    {"peasant10", "Monster"},
    {"peasant11", "Monster"},
    {"peasant12", "Monster"},
    {"peasant13", "Monster"},
    {"peasant14", "Monster"},
    {"peasant15", "Monster"},
    {"peasant16", "Monster"},
    {"peasant17", "Monster"},
    {"peasant18", "Monster"},
    {"peasant19", "Monster"},
    {"peasant2", "Monster"},
    {"peasant20", "Monster"},
    {"peasant21", "Monster"},
    {"peasant22", "Monster"},
    {"peasant3", "Monster"},
    {"peasant4", "Monster"},
    {"peasant5", "Monster"},
    {"peasant6", "Monster"},
    {"peasant7", "Monster"},
    {"peasant8", "Monster"},
    {"peasant9", "Monster"},
    {"pig", "Monster"},
    {"programmer", "Monster"},
    {"ratbuddy", "Monster"},
    {"reaver", "Monster"},
    {"rebel", "Monster"},
    {"rebel1", "Monster"},
    {"rebel2", "Monster"},
    {"rebel3", "Monster"},
    {"rebel4", "Monster"},
    {"rebel5", "Monster"},
    {"rebel6", "Monster"},
    {"revenant", "Monster"},
    {"scriptedmarine", "Monster"},
    {"sentinel", "Monster"},
    {"serpent", "Monster"},
    {"serpentleader", "Monster"},
    {"shotgunguy", "Monster"},
    {"snake", "Monster"},
    {"sorcerer1", "Monster"},
    {"sorcerer2", "Monster"},
    {"spectre", "Monster"},
    {"spidermastermind", "Monster"},
    {"stalker", "Monster"},
    {"stealtharachnotron", "Monster"},
    {"stealtharchvile", "Monster"},
    {"stealthbaron", "Monster"},
    {"stealthcacodemon", "Monster"},
    {"stealthchaingunguy", "Monster"},
    {"stealthdemon", "Monster"},
    {"stealthdoomimp", "Monster"},
    {"stealthfatso", "Monster"},
    {"stealthhellknight", "Monster"},
    {"stealthrevenant", "Monster"},
    {"stealthshotgunguy", "Monster"},
    {"stealthzombieman", "Monster"},
    {"strifebishop", "Monster"},
    {"supershotgunguy", "Monster"},
    {"templar", "Monster"},
    {"weaponsmith", "Monster"},
    {"wizard", "Monster"},
    {"wolfensteinss", "Monster"},
    {"wraith", "Monster"},
    {"wraithburied", "Monster"},
    {"zombie", "Monster"},
    {"zombieman", "Monster"},
    {"chexplayer", "Player"},
    {"chickenplayer", "Player"},
    {"clericplayer", "Player"},
    {"doomplayer", "Player"},
    {"fighterplayer", "Player"},
    {"hereticplayer", "Player"},
    {"mageplayer", "Player"},
    {"pigplayer", "Player"},
    {"strifeplayer", "Player"},
    {"powerbuddha", "Power"},
    {"powerdamage", "Power"},
    {"powerdoublefiringspeed", "Power"},
    {"powerdrain", "Power"},
    {"powerflight", "Power"},
    {"powerfrightener", "Power"},
    {"powerghost", "Power"},
    {"powerhighjump", "Power"},
    {"powerinfiniteammo", "Power"},
    {"powerinvisibility", "Power"},
    {"powerinvulnerable", "Power"},
    {"powerironfeet", "Power"},
    {"powerlightamp", "Power"},
    {"powermask", "Power"},
    {"powerminotaur", "Power"},
    {"powermorph", "Power"},
    {"powerprotection", "Power"},
    {"powerreflection", "Power"},
    {"powerregeneration", "Power"},
    {"powerscanner", "Power"},
    {"powershadow", "Power"},
    {"powerspeed", "Power"},
    {"powerstrength", "Power"},
    {"powertargeter", "Power"},
    {"powertimefreezer", "Power"},
    {"powertorch", "Power"},
    {"powerweaponlevel2", "Power"},
    {"allmap", "Powerup"},
    {"berserk", "Powerup"},
    {"blursphere", "Powerup"},
    {"computerareamap", "Powerup"},
    {"infrared", "Powerup"},
    {"invulnerabilitysphere", "Powerup"},
    {"megasphere", "Powerup"},
    {"radsuit", "Powerup"},
    {"slimeproofsuit", "Powerup"},
    {"strifemap", "Powerup"},
    {"supermap", "Powerup"},
    {"targeter", "Powerup"},
    {"ultragoggles", "Powerup"},
    {"puzzbook1", "PuzzleItem"},
    {"puzzbook2", "PuzzleItem"},
    {"puzzcweapon", "PuzzleItem"},
    {"puzzfweapon", "PuzzleItem"},
    {"puzzflamemask", "PuzzleItem"},
    {"puzzgear1", "PuzzleItem"},
    {"puzzgear2", "PuzzleItem"},
    {"puzzgear3", "PuzzleItem"},
    {"puzzgear4", "PuzzleItem"},
    {"puzzgembig", "PuzzleItem"},
    {"puzzgemblue1", "PuzzleItem"},
    {"puzzgemblue2", "PuzzleItem"},
    {"puzzgemgreen1", "PuzzleItem"},
    {"puzzgemgreen2", "PuzzleItem"},
    {"puzzgemred", "PuzzleItem"},
    {"puzzmweapon", "PuzzleItem"},
    {"puzzskull", "PuzzleItem"},
    {"beldinsring", "QuestItem"},
    {"brokenpowercoupling", "QuestItem"},
    {"coin", "QuestItem"},
    {"communicator", "QuestItem"},
    {"ear", "QuestItem"},
    {"flamethrowerparts", "QuestItem"},
    {"gold10", "QuestItem"},
    {"gold25", "QuestItem"},
    {"gold300", "QuestItem"},
    {"gold50", "QuestItem"},
    {"guarduniform", "QuestItem"},
    {"info", "QuestItem"},
    {"interrogatorreport", "QuestItem"},
    {"offeringchalice", "QuestItem"},
    {"officersuniform", "QuestItem"},
    {"proglevelender", "QuestItem"},
    {"bang4cloud", "SFX"},
    {"bat", "SFX"},
    {"bishopblur", "SFX"},
    {"bishoppainblur", "SFX"},
    {"bishoppuff", "SFX"},
    {"blackparticlefountain", "SFX"},
    {"blasteffect", "SFX"},
    {"blastersmoke", "SFX"},
    {"bloodsplash", "SFX"},
    {"bloodsplashbase", "SFX"},
    {"blueparticlefountain", "SFX"},
    {"bridgeball", "SFX"},
    {"cflamefloor", "SFX"},
    {"corpsebit", "SFX"},
    {"corpseblooddrip", "SFX"},
    {"crossbowfx4", "SFX"},
    {"decal", "SFX"},
    {"dirt1", "SFX"},
    {"dirt2", "SFX"},
    {"dirt3", "SFX"},
    {"dirt4", "SFX"},
    {"dirt5", "SFX"},
    {"dirt6", "SFX"},
    {"dirtclump", "SFX"},
    {"firedemonrock1", "SFX"},
    {"firedemonrock2", "SFX"},
    {"firedemonrock3", "SFX"},
    {"firedemonrock4", "SFX"},
    {"firedemonrock5", "SFX"},
    {"flamepuff", "SFX"},
    {"flamepuff2", "SFX"},
    {"fogpatchlarge", "SFX"},
    {"fogpatchmedium", "SFX"},
    {"fogpatchsmall", "SFX"},
    {"glassjunk", "SFX"},
    {"greenparticlefountain", "SFX"},
    {"grenadesmoketrail", "SFX"},
    {"hwaterdrip", "SFX"},
    {"holytail", "SFX"},
    {"holytailtrail", "SFX"},
    {"icechunk", "SFX"},
    {"icefxpuff", "SFX"},
    {"iceguybit", "SFX"},
    {"iceguywisp1", "SFX"},
    {"iceguywisp2", "SFX"},
    {"itemfog", "SFX"},
    {"lavasmoke", "SFX"},
    {"lavasplash", "SFX"},
    {"leaf1", "SFX"},
    {"leaf2", "SFX"},
    {"littlefly", "SFX"},
    {"loreshot2", "SFX"},
    {"magewandsmoke", "SFX"},
    {"minimissilepuff", "SFX"},
    {"minotaursmoke", "SFX"},
    {"minotaursmokeexit", "SFX"},
    {"phoenixpuff", "SFX"},
    {"pickupflash", "SFX"},
    {"playerspeedtrail", "SFX"},
    {"potterybit", "SFX"},
    {"puffy", "SFX"},
    {"purpleparticlefountain", "SFX"},
    {"redparticlefountain", "SFX"},
    {"rock1", "SFX"},
    {"rock2", "SFX"},
    {"rock3", "SFX"},
    {"rocketsmoketrail", "SFX"},
    {"rockettrail", "SFX"},
    {"sgshard0", "SFX"},
    {"sgshard1", "SFX"},
    {"sgshard2", "SFX"},
    {"sgshard3", "SFX"},
    {"sgshard4", "SFX"},
    {"sgshard5", "SFX"},
    {"sgshard6", "SFX"},
    {"sgshard7", "SFX"},
    {"sgshard8", "SFX"},
    {"sgshard9", "SFX"},
    {"slimechunk", "SFX"},
    {"slimesplash", "SFX"},
    {"sludgechunk", "SFX"},
    {"sludgesplash", "SFX"},
    {"sorcball1", "SFX"},
    {"sorcball2", "SFX"},
    {"sorcball3", "SFX"},
    {"sorcfx2", "SFX"},
    {"sorcfx2t1", "SFX"},
    {"sorcfx3explosion", "SFX"},
    {"sorcspark1", "SFX"},
    {"sorcerer2fxspark", "SFX"},
    {"sorcerer2telefade", "SFX"},
    {"soundsequenceslot", "SFX"},
    {"spark", "SFX"},
    {"spawnfire", "SFX"},
    {"strifepuff", "SFX"},
    {"strifespark", "SFX"},
    {"teleglitter1", "SFX"},
    {"teleglitter2", "SFX"},
    {"telesmoke", "SFX"},
    {"teleportfog", "SFX"},
    {"teleportswirl", "SFX"},
    {"waterdrip", "SFX"},
    {"waterdroponfloor", "SFX"},
    {"watersplash", "SFX"},
    {"watersplashbase", "SFX"},
    {"waterfallsplash", "SFX"},
    {"whiteparticlefountain", "SFX"},
    {"wraithfx2", "SFX"},
    {"wraithfx3", "SFX"},
    {"wraithfx4", "SFX"},
    {"wraithfx5", "SFX"},
    {"yellowparticlefountain", "SFX"},
    {"zarmorchunk", "SFX"},
    {"actormover", "ScriptThing"},
    {"aimingcamera", "ScriptThing"},
    {"hatetarget", "ScriptThing"},
    {"interpolationpoint", "ScriptThing"},
    {"interpolationspecial", "ScriptThing"},
    {"movingcamera", "ScriptThing"},
    {"pathfollower", "ScriptThing"},
    {"patrolpoint", "ScriptThing"},
    {"patrolspecial", "ScriptThing"},
    {"secrettrigger", "ScriptThing"},
    {"securitycamera", "ScriptThing"},
    {"ammofillup", "Token"},
    {"closedoor222", "Token"},
    {"guntraining", "Token"},
    {"healthfillup", "Token"},
    {"healthtraining", "Token"},
    {"opendoor222", "Token"},
    {"opendoor224", "Token"},
    {"questitem1", "Token"},
    {"questitem10", "Token"},
    {"questitem11", "Token"},
    {"questitem12", "Token"},
    {"questitem13", "Token"},
    {"questitem14", "Token"},
    {"questitem15", "Token"},
    {"questitem16", "Token"},
    {"questitem17", "Token"},
    {"questitem18", "Token"},
    {"questitem19", "Token"},
    {"questitem2", "Token"},
    {"questitem20", "Token"},
    {"questitem21", "Token"},
    {"questitem22", "Token"},
    {"questitem23", "Token"},
    {"questitem24", "Token"},
    {"questitem25", "Token"},
    {"questitem26", "Token"},
    {"questitem27", "Token"},
    {"questitem28", "Token"},
    {"questitem29", "Token"},
    {"questitem3", "Token"},
    {"questitem30", "Token"},
    {"questitem31", "Token"},
    {"questitem4", "Token"},
    {"questitem5", "Token"},
    {"questitem6", "Token"},
    {"questitem7", "Token"},
    {"questitem8", "Token"},
    {"questitem9", "Token"},
    {"raisealarm", "Token"},
    {"slideshowstarter", "Token"},
    {"upgradeaccuracy", "Token"},
    {"upgradestamina", "Token"},
    {"bigtree", "Vegetation"},
    {"bigtree2", "Vegetation"},
    {"chexappletree", "Vegetation"},
    {"chexbananatree", "Vegetation"},
    {"chexorangetree", "Vegetation"},
    {"chexsubmergedplant", "Vegetation"},
    {"chextallflower", "Vegetation"},
    {"chextallflower2", "Vegetation"},
    {"palmtree", "Vegetation"},
    {"pottedtree", "Vegetation"},
    {"shortbush", "Vegetation"},
    {"tallbush", "Vegetation"},
    {"torchtree", "Vegetation"},
    {"treestub", "Vegetation"},
    {"zmossceiling1", "Vegetation"},
    {"zmossceiling2", "Vegetation"},
    {"zshroomlarge1", "Vegetation"},
    {"zshroomlarge2", "Vegetation"},
    {"zshroomlarge3", "Vegetation"},
    {"zshroomsmall1", "Vegetation"},
    {"zshroomsmall2", "Vegetation"},
    {"zshroomsmall3", "Vegetation"},
    {"zshroomsmall4", "Vegetation"},
    {"zshroomsmall5", "Vegetation"},
    {"zstumpbare", "Vegetation"},
    {"zstumpburned", "Vegetation"},
    {"zstumpswamp1", "Vegetation"},
    {"zstumpswamp2", "Vegetation"},
    {"zswampvine", "Vegetation"},
    {"ztree", "Vegetation"},
    {"ztreedead", "Vegetation"},
    {"ztreegnarled1", "Vegetation"},
    {"ztreegnarled2", "Vegetation"},
    {"ztreelarge1", "Vegetation"},
    {"ztreelarge2", "Vegetation"},
    {"ztreeswamp120", "Vegetation"},
    {"ztreeswamp150", "Vegetation"},
    {"assaultgun", "Weapon"},
    {"assaultgunstanding", "Weapon"},
    {"bfg10k", "Weapon"},
    {"bfg9000", "Weapon"},
    {"beak", "Weapon"},
    {"beakpowered", "Weapon"},
    {"blaster", "Weapon"},
    {"blasterpowered", "Weapon"},
    {"bloodscourgedrop", "Weapon"},
    {"bootspoon", "Weapon"},
    {"cweapflame", "Weapon"},
    {"cweapmace", "Weapon"},
    {"cweapstaff", "Weapon"},
    {"cweapwraithverge", "Weapon"},
    {"cweaponpiece1", "Weapon"},
    {"cweaponpiece2", "Weapon"},
    {"cweaponpiece3", "Weapon"},
    {"chaingun", "Weapon"},
    {"chainsaw", "Weapon"},
    {"crossbow", "Weapon"},
    {"crossbowpowered", "Weapon"},
    {"fweapaxe", "Weapon"},
    {"fweapfist", "Weapon"},
    {"fweaphammer", "Weapon"},
    {"fweapquietus", "Weapon"},
    {"fweaponpiece1", "Weapon"},
    {"fweaponpiece2", "Weapon"},
    {"fweaponpiece3", "Weapon"},
    {"fist", "Weapon"},
    {"flamethrower", "Weapon"},
    {"gauntlets", "Weapon"},
    {"gauntletspowered", "Weapon"},
    {"goldwand", "Weapon"},
    {"goldwandpowered", "Weapon"},
    {"grenadelauncher", "Weapon"},
    {"lazdevice", "Weapon"},
    {"largezorcher", "Weapon"},
    {"mweapbloodscourge", "Weapon"},
    {"mweapfrost", "Weapon"},
    {"mweaplightning", "Weapon"},
    {"mweapwand", "Weapon"},
    {"mweaponpiece1", "Weapon"},
    {"mweaponpiece2", "Weapon"},
    {"mweaponpiece3", "Weapon"},
    {"mace", "Weapon"},
    {"macepowered", "Weapon"},
    {"mauler", "Weapon"},
    {"mauler2", "Weapon"},
    {"minimissilelauncher", "Weapon"},
    {"minizorcher", "Weapon"},
    {"minigun", "Weapon"},
    {"phasingzorcher", "Weapon"},
    {"phoenixrod", "Weapon"},
    {"phoenixrodpowered", "Weapon"},
    {"pistol", "Weapon"},
    {"plasmarifle", "Weapon"},
    {"punchdagger", "Weapon"},
    {"quietusdrop", "Weapon"},
    {"railgun", "Weapon"},
    {"rapidzorcher", "Weapon"},
    {"rocketlauncher", "Weapon"},
    {"shotgun", "Weapon"},
    {"sigil", "Weapon"},
    {"sigil1", "Weapon"},
    {"sigil2", "Weapon"},
    {"sigil3", "Weapon"},
    {"sigil4", "Weapon"},
    {"sigil5", "Weapon"},
    {"skullrod", "Weapon"},
    {"skullrodpowered", "Weapon"},
    {"snout", "Weapon"},
    {"staff", "Weapon"},
    {"staffpowered", "Weapon"},
    {"strifecrossbow", "Weapon"},
    {"strifecrossbow2", "Weapon"},
    {"strifegrenadelauncher", "Weapon"},
    {"strifegrenadelauncher2", "Weapon"},
    {"superbootspork", "Weapon"},
    {"superlargezorcher", "Weapon"},
    {"supershotgun", "Weapon"},
    {"wraithvergedrop", "Weapon"},
    {"zorchpropulsor", "Weapon"},
};

#endif
